/*
    cve-manager : CVE management tool
    Copyright (C) 2017-2024 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef     cve_import_databaseH
#define     cve_import_databaseH

#include    "libcve-manager/src/database.h"
#include    "main.h"
#include    "lists.h"

#define LINUX_KERNEL_STREAMS_TABLE      "linux_kernel_streams_import"
#define LINUX_KERNEL_FIXED_CVES_TABLE   "linux_kernel_fixed_cves_import"

//-----------------------------------------------------------------------------
// MySQL DB interface

class TSpecificDBInterface : public TDatabase{

    private:

        bool appending = false; // Insertions should be performed without
                                //  deleting everything from tables beforehand

        // Delete the DB and create an empty one with the same name
        bool ClearDB(void);
        // Create table with given name and structure described in the query
        bool CreateTable(const string &table, const string &query,
                vector<string> child_tables={});
        // Create specified tables
        bool RecreateChildTables(const vector<string> &tables);
        // Create table for CPE dictionary
        bool CreateCPETable(void);
        // Create a table for kernel streams and a table for fixed kernel CVEs
        // (first one indirectly calls another one)
        bool CreateKernelStreamsTable(void);
        bool CreateKernelFixesTable(void);
        // Create *_vul_import table for the given target (FSTEC, NVD, ...)
        bool CreateVulTable(const vector<TBranch> &branches,
                const string &target);
        // Create branch table for the given target (src or bin)
        bool CreateBranchTable(const string &branch_name, const string &target);
        // Create target table (src and bin) for each given branches
        bool CreateBranchTables(const vector<TBranch> &branches,
                const string &target);
        // Form CSV import query
        string FormCSVImportQuery(const string &csv_path,
                const string &table) const;

    public:

        // Send the specified "select" query to the database and return a
        // resulting matrix of strings and an error string (not empty if failed)
        pair<vector<vector<string>>, string> Select(const string &query,
                const size_t n_cols);
        // Select specified columns from specified table and return a resulting
        // set of strings and an error message (not empty if failed)
        pair<set<string>, string> SelectDistinct(const string &col,
                const string &table);
        // Create DB structure
        bool CreateFullStruct(const vector<TBranch> &branches,
                const vector<string> &excluded_vulsrc);
        // Call table-creation function for specified target
        bool CreateTargetTable(const vector<TBranch> &branches,
                const string &target, bool nested=false);
        // Import list of package descriptions into the DB
        bool ImportPackages(TBranch &branch);
        // Import data in CSV format into the DB
        bool ImportCSV(const vector<TBranch> &branches, const string &target,
                const string &csv_path, bool force_clean=false);
        // Update specified fields of a specified table to make the content
        // lowercase
        bool MakeLowerCaseFields(const string &table,
                const vector<string> &fields);
    };

//-----------------------------------------------------------------------------

#endif
