/*
    cve-manager : CVE management tool
    Copyright (C) 2017-2024 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef     cve_import_importerH
#define     cve_import_importerH

#include    "libcve-manager/src/conf.h"
#include    "main.h"
#include    "lists.h"
#include    "database.h"
#include    "data_preproc.h"

#define     FMT_JSON    "json"
#define     FMT_XML     "xml"

//-----------------------------------------------------------------------------
// Main class

class TImporter{

    private:

        TSpecificDBInterface db;    // DB interface for importing the data
        TCVEMConf conf;             // Program configuration
        vector<TBranch> branches;   // Branches that needs to be processed
        // {path:{vendor:{(excluded_product_names, exceptions_from_excluded)}}}
        map<string, TExcludedProducts> excluded_products;
        map<string, string> import_format = {
            {TARGET_NVD,   FMT_JSON},
            {TARGET_CPE,   FMT_XML},
            {TARGET_FSTEC, FMT_XML},
            {TARGET_KERNEL_STREAMS_SUPPORTED, FMT_JSON},
            {TARGET_KERNEL_STREAMS_EOL, FMT_JSON},
            {TARGET_KERNEL_FIXES, FMT_JSON},
            };
        set<string> displayed_notes; // Notes displayd during procedurs of import
                                     // (a same note shouldn't be display more than once)

        // Get names of all available branches
        set<string> NamesOfAllBranches(void) const;
        // Check if source with given name is excluded
        bool ExcludedVulSrc(const string &name) const;
        // Fill a map of excluded products from a specified list
        bool GetExcludedProducts(const string &path);
        // Recreate structure of whole DB
        bool RecreateDBStruct(void){
            return db.CreateFullStruct(branches, conf.ExclVulSrc());
            }
        // Remove non-printable chars from the specified text file and replace
        // all unicode-spaces contained in it with the ASCII space char
        bool HandleDisallowedSymbols(const string &file_full_path);

    public:

        // Configure the program
        bool Configure(void);
        // Recreate the DB and import everything that could be imported
        bool PrepareDB(void);
        // Call create-table-function for a specified target
        bool CreateTargetTable(const string &target){
            return db.CreateTargetTable(branches, target);
            }
        // Import lists of packages for all considered branches
        bool ImportPackages(const set<string> &selected_branches={});
        // Import data from XML or JSON file
        bool ImportXmlOrJson(const string &target, const string &file_path="",
                const bool force_clean=false);
        bool ImportXmlOrJson(const string &target, const set<string> &file_paths,
                const bool force_clean);
        // Create missing entries for those NVD vulnerabilities that are present
        // in the <branch>_cve_fixes thables and fstec_vul_import table
        // (column "cve_ids")
        bool SearchMissingCVEs(void);
    };

//-----------------------------------------------------------------------------

#endif
